from ocp_resources.resource import Resource


class ImageContentSourcePolicy(Resource):
    """
    ICSP object, inherited from Resource.
    """

    api_group = Resource.ApiGroup.OPERATOR_OPENSHIFT_IO
